/************************************************************************
	main.c

    PWM Led Brightness Demonstration 2
    Copyright (C) 2010 Simon Inns

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

	Email: simon.inns@gmail.com

************************************************************************/

// Global includes
#include <htc.h>

// PIC 18F4550 fuse configuration:
// Config word 1 (Oscillator configuration)
__CONFIG(1, USBPLL & IESODIS & FCMDIS & HSPLL & CPUDIV1 & PLLDIV5);
// Config word 2
__CONFIG(2, VREGEN & PWRTDIS & BORDIS & WDTDIS);
// Config word 3
__CONFIG(3, PBDIGITAL & LPT1DIS & MCLREN);
// Config word 4
__CONFIG(4, XINSTDIS & STVRDIS & LVPDIS & ICPORTDIS & DEBUGDIS);
// Config word 5, 6 and 7 (protection configuration)
__CONFIG(5, UNPROTECT);
__CONFIG(6, UNPROTECT);
__CONFIG(7, UNPROTECT);

// Fosc frequency (48 Mhz)
#define _XTAL_FREQ 48000000

// Define the LED port
#define LED0 RA0

// Globals for PWM
unsigned char pwmCounter = 0;
unsigned char ledActualBrightness = 0;
unsigned char ledTargetBrightness = 0;

unsigned char fadeCounter = 0;

// Low priority interrupt procedure
void interrupt low_priority lpHandler(void)
{
	// Is this timer0 interrupting?
	if (TMR0IF)
	{
		// Perform the PWM brightness control
		if (ledActualBrightness > pwmCounter) LED0 = 1; else LED0 = 0;

		pwmCounter++;
		if (pwmCounter > 19) pwmCounter = 0;
		
		// Perform fading control
		if (ledTargetBrightness >= ledActualBrightness) ledActualBrightness = ledTargetBrightness;
		else
		{
			fadeCounter++;
			if (fadeCounter == 24)
			{
				ledActualBrightness--;
				fadeCounter = 0;
			}	
		}	
		
		// Get ready for the next interrupt
		TMR0L = 255 - 187;	// Reset the timer0 counter
		TMR0IF = 0;			// Clear the timer0 interrupt flag
	}
}

void main(void)
{
	// Configure ports as inputs (1) or outputs(0)
	TRISA = 0b00000000;
	TRISB = 0b00000000;
	TRISC = 0b00000000;
	TRISD = 0b00000000;
	TRISE = 0b00000000;

	// Clear all ports
	PORTA = 0b00000000;
	PORTB = 0b00000000;
	PORTC = 0b00000000;
	PORTD = 0b00000000;
	PORTE = 0b00000000;

	// Disable the USB device
	UCON = 0b00000000;
	UCFG = 0b00001000;

	// Turn off analogue ports
	ADCON1 = 0x0F;
	
	// Enable interrupts with priority
	IPEN = 1;

	// Set up timer0
	TMR0IP = 0;			// Set timer0 interrupt to low priority
	TMR0IF = 0;			// Clear the timer0 interrupt flag
	TMR0L = 255 - 187;	// Reset the timer0 counter

	T0CON = 0b11000101; // Timer0 on, 8-bit and 1:64 prescaler
	TMR0IE = 1;			// Enable the timer0 interrupt

	// Enable interrupts
	GIEH = 1;			// Global enable all high priority interrupts
	GIEL = 1;			// Global enable all low priority interrupts

	while(1)
	{
		// On full brightness
		ledTargetBrightness = 19;
		
		// Pause for a bit
		for (int pause = 0; pause < 10000; pause++) __delay_us(200);
		
		// Off
		ledTargetBrightness = 0;
		
		// Pause for a bit
		for (int pause = 0; pause < 10000; pause++) __delay_us(200);
	}
}
		
		